#
# Test MultiEvent content type initialization
#
import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from DateTime import DateTime

from Products.MultiEvent.tests import MultiEventTestCase
from Products.MultiEvent.utilities import *

class TestMultiEventCreation(MultiEventTestCase.MultiEventTestCase):
    """Ensure content types can be created, edited and updated"""

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.folder.invokeFactory('MultiEvent', id="me1", title="Plone Course",)
        self.me1 = getattr(self.folder, 'me1')

    def testCreateMultiEvent(self):
        self.failUnless('me1' in self.folder.objectIds())

    def testEditMultiEvent(self):

        self.me1.setTitle('Plone Course')
        self.me1.setDescription('A description')
        self.me1.setText('<p>Body text</p>')
        self.me1.setLocation('Pisa, IT')
        self.me1.setUpcomingEvents([{ 'location' : 'Rome, IT',
                              'startDate' : '1/1/2008',
                              'endDate' : '2/1/2008',}, ]
                          )

        self.assertEqual(self.me1.Title(), 'Plone Course')
        self.assertEqual(self.me1.Description(), 'A description')
        self.assertEqual(self.me1.getText(), '<p>Body text</p>')
        self.assertEqual(self.me1.getLocation(), 'Pisa, IT')
        self.assertEqual(self.me1.getUpcomingEvents(),
                           ({ 'location' : 'Rome, IT',
                              'startDate' : '1/1/2008',
                              'endDate' : '2/1/2008',}, ))

    def testValidation(self):
        request = {}
        request['UpcomingEvents'] = [
                  { 'location' : 'Rome, IT',
                     'startDate' : 'March 2008',
                     'endDate' : '2/1/2008',},
                ]

        errors = {}
        self.me1.post_validate(request, errors)
        self.failUnless('UpcomingEvents' in errors.keys())

        request = {}
        request['UpcomingEvents'] = [
                  { 'location' : 'Rome, IT',
                     'startDate' : '1/1/2007',
                     'endDate' : '2/1/2007',},
                ]

        errors = {}
        self.me1.post_validate(request, errors)
        self.failUnless('UpcomingEvents' in errors.keys())

    def testUpdate(self):

        now = DateTime()

        # now before current date: unchange
        current = now+1
        self.me1.setStartDate(current)
        self.me1.setEndDate(current+1)
        self.me1.setLocation('Pisa, IT')

        future_startdate = now + 10
        future_enddate = now + 11
        self.me1._addEvent('Milan, IT', date_in_formatted_string_format(future_startdate), date_in_formatted_string_format(future_enddate))
        self.me1.updateEvent()

        self.failUnless(self.me1.getLocation()=='Pisa, IT')
        self.failUnless(self.me1.start().Date()==current.Date())

        # now after current date: update
        initial_stardate = now - 10
        initial_enddate = now - 9
        self.me1.setStartDate(initial_stardate)
        self.me1.setEndDate(initial_enddate)
        self.me1.updateEvent()

        self.failUnless(self.me1.start().Date()==future_startdate.Date())
        self.failUnless(self.me1.getLocation()=='Milan, IT')

        past_events = self.me1.getPastEvents()
        self.failUnless(len(past_events)==1)
        self.failUnless(past_events[0]['startDate']==date_in_formatted_string_format(initial_stardate))
        self.failUnless(past_events[0]['endDate']==date_in_formatted_string_format(initial_enddate))
        self.failUnless(past_events[0]['location']=='Pisa, IT')


    def testUpcomingEventsOrder(self):
        now = DateTime()

        self.me1._addEvent('Milan, IT',
                           date_in_formatted_string_format(now+6),
                           date_in_formatted_string_format(now+7))
        self.me1._addEvent('Pisa, IT',
                           date_in_formatted_string_format(now+4),
                           date_in_formatted_string_format(now+5))
        self.me1._addEvent('Florence, IT',
                           date_in_formatted_string_format(now+2),
                           date_in_formatted_string_format(now+3))
        self.me1._addEvent('Rome, IT',
                           date_in_formatted_string_format(now),
                           date_in_formatted_string_format(now+1))

        self.me1.processForm()
        events = self.me1.getUpcomingEventsAsDateTime()
        self.failUnless(events[0]['startDate'] < events[1]['startDate'])
        self.failUnless(events[1]['startDate'] < events[2]['startDate'])
        self.failUnless(events[2]['startDate'] < events[3]['startDate'])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMultiEventCreation))
    return suite

if __name__ == '__main__':
    framework()