#
# Test our Press Room installation
#

import os, sys
import string
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.MultiEvent.tests import MultiEventTestCase

class TestInstallation(MultiEventTestCase.MultiEventTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        self.css        = self.portal.portal_css
        self.kupu       = self.portal.kupu_library_tool
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.factory    = self.portal.portal_factory
        self.workflow   = self.portal.portal_workflow
        self.properties = self.portal.portal_properties
        self.atct_tool  = self.portal.portal_atct

        self.metaTypes = ('MultiEvent',)

    def testSkinLayersInstalled(self):
        self.failUnless('multievent' in self.skins.objectIds())

    def testTypesInstalled(self):
        for t in self.metaTypes:
            self.failUnless(t in self.types.objectIds())

    def testPortalFactorySetup(self):
        for t in self.metaTypes:
            if t != 'MultiEvent':
                self.failUnless(t in self.factory.getFactoryTypes())

    def testTypesNotSearched(self):
        # PR originally wanted Press Contacts to be unsearchable, but that doesn't make
        # sense -- people want to be able to find press contacts as quickly as possible
        # this test now confirms only that PressContacts are indeed searchable
        types_not_searched = self.properties.site_properties.getProperty('types_not_searched')
        self.failIf('MultiEvent' in types_not_searched)


class TestContentCreation(MultiEventTestCase.MultiEventTestCase):
    """Ensure content types can be created and edited"""

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.folder.invokeFactory('MultiEvent', 'me1')
        self.me1 = getattr(self.folder, 'me1')

    def testCreateMultiEvent(self):
        self.failUnless('me1'in self.folder.objectIds())

    def testEditMultiEvent(self):

        self.me1.setTitle('Plone Course')
        self.me1.setDescription('A description')
        self.me1.setText('<p>Body text</p>')
        self.me1.setLocation('Pisa, IT')
        self.me1.setUpcomingEvents([{ 'location' : 'Rome, IT',
                              'startDate' : '1/1/2008',
                              'endDate' : '2/1/2008',}, ]
                          )

        self.assertEqual(self.me1.Title(), 'Plone Course')
        self.assertEqual(self.me1.Description(), 'A description')
        self.assertEqual(self.me1.getText(), '<p>Body text</p>')
        self.assertEqual(self.me1.getLocation(), 'Pisa, IT')
        self.assertEqual(self.me1.getUpcomingEvents(),
                           ({ 'location' : 'Rome, IT',
                              'startDate' : '1/1/2008',
                              'endDate' : '2/1/2008',}, ))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestContentCreation))
    return suite

if __name__ == '__main__':
    framework()